import java.lang.*;

/*
-------------------------------------------------------------------------------
Model presa-depredador (sense normalitzar). Inclou model discret i continu
-------------------------------------------------------------------------------
Dades: dt (duracio del pas de temps), imax (passos que es vol calcular), jmax
       (iteracions per resoldre les eqs. diferencials), nx0 (nombre inicial
       preses), ny0 (nombre inicial depredadors), a (parametre alfa), b
       (parametre beta), Rx (factor de creixement presa), delt (mortalitat
       depredador), Kx (capacitat presa)
Resultats: Fitxer "volterra.dat". Cada fila es un pas de temps. La primera
           columna es el temps, la segona i la tercera son el nombre de preses
           i depredadors, respectivament, segons el model discret. La quarta
           i la cinquena son el mateix, pero amb el model continu.
Dades de prova: dt=0.1 imax=2 jmax=1 nx0=200 ny0=50 alfa=0.1 beta=0.1
                Rx=2.0 delt=0.1 Kx=1000 ==>
                0.000000, 200.000003, 50.000001, 200.000003, 50.000001;
                0.100000, 310.403556, 45.050003, 359.499991, 45.049999;
                0.200000, 458.074749, 40.614922, 588.949978, 40.625975;
Observacions: Si compila malament, prova amb l'opcio -lm
-------------------------------------------------------------------------------
*/

public class Lotka2Calculs {
	double dt,nx0,ny0,a,b,Rx,delt,Kx,temps[],trajx[],trajy[],trajxc[],trajyc[];
	int imax;

	Lotka2Calculs(float dt0,int imax0,float nx00,float ny00,float a0,float b0,float Rx0,float delt0,float Kx0) {
		double dx,dy,x0,y0,t0,x,y,t,xf,yf,tf,Xf,Yf,aa,bb,dd,xm,ym,tm,Qx,rx,X,Y;
		int i,j,jmax;

		dt = (double) dt0;
		nx0 = (double) nx00;
		ny0 = (double) ny00;
		a = (double) a0;
		b = (double) b0;
		Rx = (double) Rx0;
		delt = (double) delt0;
		Kx = (double) Kx0;
		imax = imax0;
		jmax = 1;
		temps = new double[imax];
		trajx = new double[imax];
		trajy = new double[imax];
		trajxc = new double[imax];
		trajyc = new double[imax];

/* Hipotesi 1 */

		rx=Math.log(Rx)/dt;

/* Hipotesi 2 */

		Qx=Kx*((1.0/(Rx-1))+1.0);
		aa=a/dt*Kx/Qx;
		bb=b;
		dd=delt/dt;

/* Normalitzacio */

		x0=nx0/Kx ;
		y0=ny0/Kx ;
		X=nx0/Qx ;
		Y=ny0/Qx ;

		t0=0;
		x=x0;
		y=y0;
		t=t0;
		xf=x0;
		yf=y0;
		temps[0]=t0;
		trajx[0]=x0*Kx;
		trajy[0]=y0*Kx;
		trajxc[0]=X*Qx;
		trajyc[0]=Y*Qx;

		for(i=0;i<imax;i++) {
			tm=t+dt/2.0;
			for(j=0;j<jmax;j++) {
				xm=(x+xf)/2.0;
				ym=(y+yf)/2.0;
				dx=dt*xm*(rx*(1.0-xm)-aa*ym);
				dy=dt*ym*(aa*bb*xm-dd);
				xf=x+dx;
				yf=y+dy;
			}
			x=xf;
			y=yf;
			t=t+dt;
			Xf=Rx*(1.0-X)*X-a*X*Y;
			Yf=a*b*X*Y+(1.0-delt)*Y;
			X=Xf;
			Y=Yf;
			temps[i]=t;
			trajx[i]=x*Kx;
			trajy[i]=y*Kx;
			trajxc[i]=X*Qx;
			trajyc[i]=Y*Qx;
		}
	}
}
